/*
 * Software for Interactive Project Pool Game
 * Columbia University. New York, 2008
 * Authors:
 *   Abdulhamid Ghandour
 *   Thomas John
 *   Jaime Peretzman
 *   Bharadwaj Vellore
 * 
 * Desc: LCD implementation borrowed from code by Prof. Stephen Edwards,
 * Columbia University
 */
#ifndef _UI_H_
#define _UI_H_

#include "system.h"
#include "io.h"
#include "ball.h"

//  LCD Module 16*2
#define lcd_write_cmd(base, data)              IOWR(base, 0, data)
#define lcd_read_cmd(base)                     IORD(base, 1)
#define lcd_write_data(base, data)             IOWR(base, 2, data)
#define lcd_read_data(base)                    IORD(base, 3)

#define IOWR_LED_DATA(base, offset, data)      \
                               IOWR_16DIRECT(base, (offset) * 2, data) 
#define IORD_LED_DATA(base, offset)            \
                               IORD_16DIRECT(base, (offset) * 2)
#define IOWR_LED_SPEED(base, data)             \
                               IOWR_16DIRECT(base + 32, 0, data)
#define IORD_FLAG (base, offset)               \
                               IORD_16DIRECT(base, (offset) * 2)

#define PLAY_SOUND             IOWR_32DIRECT(SOUNDDRIVER_BASE, 0, 1)
#define HEXWRITE(reg,data)     IOWR_32DIRECT(UICONTROL_BASE, (reg) * 4, data)     

#define HEX0(data)             HEXWRITE(0, (data))
#define HEX1(data)             HEXWRITE(1, (data))
#define HEX2(data)             HEXWRITE(2, (data))
#define HEX3(data)             HEXWRITE(3, (data))
#define HEX4(data)             HEXWRITE(4, (data))
#define HEX5(data)             HEXWRITE(5, (data))
#define HEX6(data)             HEXWRITE(6, (data))
#define HEX7(data)             HEXWRITE(7, (data))

#define CALIBRATION_REQUESTED  IORD_32DIRECT(UICONTROL_BASE, 8 * 4)
#define CALIBRATED             IOWR_32DIRECT(UICONTROL_BASE, 8 * 4, 1)
#define NEW_GAME_REQUESTED     IORD_32DIRECT(UICONTROL_BASE, 9 * 4)
#define STARTED_GAME           IOWR_32DIRECT(UICONTROL_BASE, 9 * 4, 1)

extern unsigned long sevensegment[];

void LCD_Init();
void LCD_Show_Text(char* Text);
void LCD_Line2();
void initPointsDisplay();

#endif /* _UI_H_ */
