library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity calibration is
  
  port (
    reset      : in std_logic;
    clk	       : in std_logic; 
    valid_green : in std_logic;
    end_row 	: in std_logic;
    end_frame   : in std_logic;
    green_pixel_value : in unsigned (9 downto 0);
--    green_column_thr   : in unsigned (9 downto 0);
--    green_row_thr      : in unsigned (9 downto 0);
    repos       : out unsigned (6 downto 0) := "1000000";
    x_1         : out unsigned  (10 downto 0) := "00000000000";
    y_1         : out unsigned  (10 downto 0) := "00000000000";
    x_2         : out unsigned  (10 downto 0) := "00000000000";
    y_2         : out unsigned  (10 downto 0) := "00000000000";
    calibration_on : in std_logic;
    threshold   : in unsigned (9 downto 0);
    leds : out unsigned(6 downto 0)
  );

end calibration;

architecture rtl of calibration is
  
  -- signals for calibration
  signal row_counter		: unsigned (10 downto 0):= "00000000000";
  signal green_column_count : unsigned (10 downto 0):= "00000000000"; 
  signal green_row_count    : unsigned (10 downto 0):= "00000000000";
  signal green_column_thr   : unsigned (9 downto 0) := "0001111101";
  signal green_row_thr      : unsigned (9 downto 0) := "0110010000";
  signal green_color_thr    : unsigned (9 downto 0);
  signal green_x1_thr       : unsigned (9 downto 0) := "0001101000";
  signal black_column_thr   : unsigned (2 downto 0) := "011";
  signal black_column_count : unsigned (2 downto 0) := "000";
  signal temp_active_row    : unsigned (2 downto 0) := "000";
  signal temp_changes_sig   : unsigned (1 downto 0) := "00";
  signal black_first_count  : unsigned (4 downto 0) := "00000";
  signal first_row_black    : std_logic := '0';
  signal temp_left_column   : std_logic := '0';
  signal temp_green_row	    : std_logic := '0';
  signal temp_right_column  : std_logic := '0';
  signal flag               : std_logic := '0';
  signal flag2              : std_logic := '0';
  signal start_flag         : std_logic := '0';
  signal temp_x_1           : unsigned  (10 downto 0) := "00000000000";
  signal temp_y_1           : unsigned  (10 downto 0) := "00000000000";
  signal temp_x_2           : unsigned  (10 downto 0) := "01111111111";
  signal temp_y_2           : unsigned  (10 downto 0) := "00000000000";
  signal column_counter     : unsigned  (10 downto 0) := "00000000000";
  signal active_row         : unsigned (2 downto 0) := "000";
  signal changes_sig        : unsigned (1 downto 0) := "00";
  signal cam_repos          : unsigned (6 downto 0) := "1000000";
  signal debug_x_counter    : unsigned (10 downto 0) := "00000000000";
  signal debug_x_max        : unsigned (10 downto 0) := "00000000000";
----------------
begin  
  green_color_thr <= threshold;

  Calib : process (clk)
  begin
    if rising_edge(clk) then
      if reset = '1' then
        row_counter <= (others=>'0');
        green_column_count <= (others =>'0');
        green_row_count    <= (others =>'0');
        black_column_count <= (others =>'0');
        temp_active_row    <= (others =>'0');
        temp_changes_sig   <= (others =>'0');
        black_first_count  <= (others =>'0');	
        temp_green_row 	   <= '0';
        temp_left_column   <= '0';
        temp_right_column  <= '0';
        flag               <= '0'; 
        flag2               <= '0'; 
        first_row_black	   <= '0';
        start_flag	  	   <= '0';		 
        x_1				   <= (others =>'0');
        x_2		  	       <= (others =>'0');
        y_1                <= (others =>'0');
        y_2                <= (others =>'0');
        temp_x_1           <= (others =>'0');
        temp_y_1           <= (others =>'0');
        temp_x_2           <= "01111111111";
        temp_y_2           <= (others =>'0');
        column_counter     <= (others =>'0');
        active_row   	   <= (others =>'0');
        changes_sig        <= (others =>'0');	
        cam_repos          <= "1000000";

      elsif valid_green = '1' and calibration_on = '1' then
       --  if start_flag = '1' then
        --cam_repos <= (others => '1');
        if green_pixel_value > green_color_thr then
          green_column_count <= green_column_count + 1;
          black_column_count <= (others =>'0');
		else
          black_column_count <= black_column_count + 1;
		end if;

		column_counter <= column_counter + 1;
		debug_x_counter <= debug_x_counter + 1; -- ********DEBUG**********

        if black_column_count >= black_column_thr then		
          if green_column_count >= green_column_thr then
            temp_right_column <= '1';
          elsif green_column_count < green_column_thr then
            temp_left_column <= '1';
            green_column_count <= (others =>'0');
          end if;	
        end if;
		
		if green_column_count >= green_column_thr then
          temp_green_row <= '1';
          if green_row_count >= green_row_thr and flag ='0' then
            if temp_x_1 < column_counter - green_column_thr and flag2 = '0'then
              temp_x_1 <= column_counter - green_column_thr;
                flag2 <='1';
            end if;
			if black_column_count >= black_column_thr  then
				temp_x_2 <= column_counter - black_column_thr;
				flag <='1';
			end  if;
--            if black_column_count >= black_column_thr  then		
--              if temp_x_2 < column_counter - black_column_thr and flag ='0' then
--                temp_x_2 <= column_counter - black_column_thr;
--                flag <='1';
--              end if;
--            end if;
          end if;
		end if;

        if black_first_count > 1 then
          first_row_black <='1';
          black_first_count<=(others=>'0');
        end if;
      end if; -- end of valid green
									
      if end_row = '1' then
        if temp_green_row ='1' then
          green_row_count <= green_row_count +1;
        end if;
         
--**************DEBUG***************
debug_x_counter <= (others => '0');

		if debug_x_counter > debug_x_max then
			debug_x_max <= debug_x_counter;
			
		end if;
--**********DEBUG END***************
        if row_counter < 20 and temp_green_row ='0' then
          black_first_count <= black_first_count +1;
        end if;

        if green_row_count >= green_row_thr and green_row_count < green_row_thr+2 then
          if first_row_black ='1' then
            temp_y_1		    <= row_counter - green_row_thr + 1;
            temp_changes_sig(0) <= '1';
          end if;
          temp_active_row(2)    <= temp_left_column;
          temp_active_row(1)    <= temp_green_row ;
          temp_active_row(0)    <= temp_right_column;	
        end if;

        if green_row_count >= green_row_thr and temp_green_row ='0'  then
          temp_y_2				<= row_counter - 1;
          temp_changes_sig(1)   <= '1';
          green_row_count		<= (others =>'0');
		end if;
        
        column_counter     <= (others =>'0');
        --flag			   <= '0'; 
        temp_green_row 	   <= '0';
        temp_left_column   <= '0';
        temp_right_column  <= '0';
        green_column_count <= (others =>'0');
        black_column_count <= (others =>'0');
        row_counter <= row_counter + 1;
	  end if;
	--	end if; -- end of start flag
	
      if end_frame = '1' then
		debug_x_max <= (others => '0');  --******DEBUG************
		debug_x_counter <= (others => '0');--******DEBUG************
        if temp_changes_sig = "00" then
          if temp_active_row = "010" then
            cam_repos <= "1010000";
          elsif temp_active_row = "011" then
            cam_repos <= "1010010";
          elsif temp_active_row = "110" then
            cam_repos <= "1010001";
          elsif temp_active_row = "111" then
            cam_repos <= "1010000";
          else
            cam_repos <= "1000000";
		  end if;
		elsif temp_changes_sig = "01" then
		  if temp_active_row = "010" then
            cam_repos <= "1010100";
          elsif temp_active_row = "011" then
            cam_repos <= "1000110";
          elsif temp_active_row = "110" then
            cam_repos <= "1000101";
          elsif temp_active_row = "111" then
            cam_repos <= "1000100";
          else
            cam_repos <= "1000000";
          end if;
		elsif temp_changes_sig = "10" then
          if temp_active_row = "010" then
            cam_repos <= "1011000";
          elsif temp_active_row = "011" then
            cam_repos <= "1001010";
          elsif temp_active_row = "110" then
            cam_repos <= "1001001";
          elsif temp_active_row = "111" then
            cam_repos <= "1001000";
          else
            cam_repos <= "1000000";
          end if;
		elsif temp_changes_sig = "11" then
          if temp_active_row = "010" then
            cam_repos <= "1010000";
          elsif temp_active_row = "011" then
            cam_repos <= "1000010";
          elsif temp_active_row = "110" then
            cam_repos <= "1000001";
          elsif temp_active_row = "111" then
            cam_repos <= "0000000";
          else
            cam_repos <= "1000000";
          end if;
		end if;

        --cam_repos <= "1111111";
	    active_row		   <= temp_active_row;
	    changes_sig		   <= temp_changes_sig;
	    x_1				   <= temp_x_1;
	    y_1				   <= temp_y_1;
	    x_2				   <= temp_x_2;
	    y_2				   <= temp_y_2;
	    temp_x_1		   <= (others =>'0');
	    temp_y_1		   <= (others =>'0');
        temp_x_2           <= "01111111111";
	    temp_y_2		   <= (others =>'0');
	    temp_changes_sig   <= (others=>'0');
	    row_counter 	   <= (others=>'0');
	    column_counter     <= (others =>'0');
	    black_first_count  <= (others =>'0');
	    temp_green_row 	   <= '0';
	    temp_left_column   <= '0';
	    temp_right_column  <= '0';
	    flag			   <= '0'; 
	    flag2              <= '0'; 
	    first_row_black	   <= '0';
     	temp_active_row    <= (others =>'0');
	    green_column_count <= (others =>'0');
	    green_row_count    <= (others =>'0');
	    black_column_count <= (others =>'0');
	    start_flag 		   <= '1';
      end if;  
	  if  calibration_on = '0' then
		cam_repos 	   <= "1000000";
		x_1				   <= (others =>'0');
        x_2		  	       <= (others =>'0');
        y_1                <= (others =>'0');
        y_2                <= (others =>'0');
	  end if;					
--      if chipselect = '1' then
--        if write = '1' then		
--          if address =  "000" then
--            calibration_on <= writedata(0);
--            cam_repos 	   <= "1000000";
--          end if;			
--        end if;
--
--        if read = '1' then	
--          if address = "001" then
--            readdata(6 downto 0) <= cam_repos;
--          elsif address = "010" then
--            readdata(9 downto 0) <= x_1;
--          elsif address = "011" then
--            readdata(9 downto 0) <= y_1;
--          elsif address = "100" then
--            readdata(9 downto 0) <= x_2;
--          elsif address = "101" then
--            readdata(9 downto 0) <= y_2;
--          end if;
--        end if;
--      end if;-- end of chipslect
      leds <= cam_repos;
      repos <= cam_repos;
    end if;
  end process Calib;
end rtl;
