library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity imagecropper is
  port (
    clk	            : in std_logic; 
    valid_green_in  : in std_logic;
    valid_green_out : out std_logic;
    end_row_in 	    : in std_logic;
    end_row_out     : out std_logic; 
    end_frame       : in std_logic;
    crop_start_x    : in unsigned  (10 downto 0);
    crop_end_x      : in unsigned  (10 downto 0);
    crop_start_y    : in unsigned  (10 downto 0);
    crop_end_y      : in unsigned  (10 downto 0)
  );
end imagecropper;

architecture rtl of imagecropper is
  signal xcount: unsigned (10 downto 0) := (others => '0');
  signal ycount: unsigned (10 downto 0) := (others => '0');
  signal crop_start_x_sig: unsigned  (10 downto 0) := (others => '0');
  signal crop_end_x_sig  : unsigned  (10 downto 0) := (others => '1');
  signal crop_start_y_sig: unsigned  (10 downto 0) := (others => '0');
  signal crop_end_y_sig  : unsigned  (10 downto 0) := (others => '1');
begin

  control : process(clk)
  begin
    if rising_edge(clk) then
      if end_frame = '1' then
        crop_start_x_sig <= crop_start_x;
        crop_end_x_sig <= crop_end_x;
        crop_start_y_sig <= crop_start_y;
        crop_end_y_sig <= crop_end_y;
      end if;
    end if;
  end process control;

  xcounter : process (clk)
  begin
    if rising_edge (clk) then
      if end_row_in = '1' then
         xcount <= (others => '0');
      else
        if valid_green_in = '1' then
          xcount <= xcount + 1;
        end if;
      end if;
    end if;
  end process xcounter;

  ycounter : process (clk)
  begin
    if rising_edge (clk) then
      if end_frame = '1' then
        ycount <= (others => '0');
      else
        if end_row_in = '1' then
          ycount <= ycount + 1;
        end if;
      end if;
    end if;
  end process ycounter;

  valid_green_out <= valid_green_in when
    (((xcount >= crop_start_x_sig) and (xcount <= crop_end_x_sig)) and 
     ((ycount >= crop_start_y_sig) and (ycount <= crop_end_y_sig))) else '0';
  end_row_out <= end_row_in when 
    ((ycount >= crop_start_y_sig) and (ycount <= crop_end_y_sig)) else '0';

end rtl;